#!/usr/bin/perl
#### $Id: gen_iplist.pl,v 1.3 2003/09/24 17:12:16 bzizou Exp $ ####
#
# This script makes a list of all the ip addresses contained into a 
# network. Option -A allows to agregate networks of B or C classes
# when all the ip are to be listed.
# Other option are telling the way the output is formated to adapt
# to a particular use (sendmail, http, ...)
#
# This script is part of the SLIS Project initiated by the CARMI-Internet
# (Acadmie de Grenoble - France 38).
# Ce script fait partie du projet SLIS dmarr par le CARMI-Internet
# (Acadmie de Grenoble - France 38).
#
# SLIS : Serveur de communications Linux Pour l'Internet Scolaire.
# Copyright (C) 1998-2003 Bruno Bzeznik
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#    You should have received a copy of the GNU General Public License
#    along with this program (For example ./COPYING);
#    if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
#    Cambridge, MA 02139, USA.
#
# Please send all comments and bug reports by electronic mail to:
#   Bruno Bzeznik <Bruno@ac-grenoble.fr>
# or to <slis@ac-grenoble.fr>
#
# Envoyez vos suggestions et reports de bugs par e-mail 
#   Bruno Bzeznik <Bruno@ac-grenoble.fr>
# ou  <slis@ac-grenoble.fr>

use NetAddr::IP;
use Getopt::Std;

# Get the switches
getopts('i:m:saFhnp:b:');

# The help
if ($opt_h) {
  print "Usage: gen_iplist.pl [-s|-a|-n|-h] [-F] [-p|-b <string>] -i <ip> -m <netmask>\n";
  print "It creates the list of all the ip addresses contained into a network\n";
  print " -i <ip> : The ip of the network\n";
  print " -m <netmask> : The netmask of the network\n";
  print " -s : Output for a Sendmail access.db file\n";
  print " -a : Output for an Apache access file\n";
  print " -n : Output for the setup.schedule SLIS script\n";
  print " -F : Aggregates IPs into full A, B or C classes when possible\n";
  print " -p <string>: A string to print after each ip (like \"RELAY\")\n";
  print " -b <string>: A string to print before each ip (like \"allow from\")\n";
  exit 0;
}

# The tests
if ( !$opt_i || !$opt_m) {
  print "Both -i and -m arguments must be present. Try -h for help.\n";
  exit 2;
} 
if ($opt_s+$opt_a+$opt_n != 1) {
  print "One (and only one) of the -a -s or -n option must be given.\n";
  print "Try -h for help.\n";
  exit 3;
}

# Space definition
$space=new NetAddr::IP $opt_i, $opt_m or die  
  "$opt_i/$opt_m is not a valid network!\n";

# The strings
if ($opt_p) { $string_after="\t$opt_p";} else { $string_after=""; }
if ($opt_b) { $string_before="$opt_b\t";} else { $string_before=""; }
if ($opt_a) { $apache_dot=".";} else { $apache_dot=""; }

if (!$opt_n) {
  # With no agregation (without -F switch), just list all the ips.
  # If the mask is longer than 24 bits, it's the same case.
  # The way used here is optimized for memory use 
  #   (see http://search.cpan.org/src/LUISMUNOZ/NetAddr-IP-3.14/tutorial.htm)
  if (!$opt_F || $space->masklen > 24) {
    for (my $ip = $space->first;
           $ip <= $space->last;
           $ip++) {
        print $string_before.$ip->addr."$string_after\n";
    }
    exit 0;
  }

  # With agregation (with the -F switch) it's a bit different.
  else {

    # Mask lenght is between 16 and 24: we split into full C classes
    if ($space->masklen > 16 and $space->masklen <= 24) {
      foreach ($space->split(24)) {
        ($a,$b,$c,$d)=split(/\./);
        print $string_before."$a.$b.$c".$apache_dot.$string_after."\n";
      }
      exit 0;
    }

    # Mask lenght is between 8 and 16: we split into full B classes
    elsif ($space->masklen > 8 and $space->masklen <= 16) {
      foreach ($space->split(16)) {
        ($a,$b,$c,$d)=split(/\./);
        print $string_before."$a.$b".$apache_dot.$string_after."\n";
      }
      exit 0;
    }

    # Mask lenght is smaller than 8: we split into full A classes
    elsif ($space->masklen <= 8 ) {
      foreach ($space->split(8)) {
        ($a,$b,$c,$d)=split(/\./);
        print $string_before."$a".$apache_dot.$string_after."\n";
      }
      exit 0;
    }
  }
}

# The setup_schedules type is special
else {

  # Mask lenght is between 23 and 29: we split into /29 networks
  if ($space->masklen > 23 and $space->masklen <= 29) {
    foreach ($space->split(29)) {
      print $string_before.$_->addr()." ".$_->mask().$string_after."\n";
    }
    exit 0;
  }

  # Mask lenght is between 0 and 23: we split into /masklen+5 networks (to have 32 subnets)
  elsif ($space->masklen > 0 and $space->masklen <= 23) {
    foreach ($space->split($space->masklen+5)) {
      print $string_before.$_->addr()." ".$_->mask().$string_after."\n";
    }
    exit 0;
  }

  # Else, error
  else {
    print "Sorry, we do not manage this kind of networks!\n";
  }
}
